@@ -1,6 +1,7 @@
 /*****************************************************************************
- * Copyright (C) 2005 Eliot
- * Authors: Olivier Teuliere  <ipkiss@via.ecp.fr>
+ * Eliot
+ * Copyright (C) 2005-2007 Olivier Teulière
+ * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,89 +20,88 @@
 
 #include <iomanip>
 #include <wctype.h>
+
+#include "freegame.h"
 #include "dic.h"
 #include "tile.h"
 #include "rack.h"
 #include "round.h"
+#include "move.h"
 #include "pldrack.h"
 #include "results.h"
 #include "player.h"
 #include "ai_player.h"
-#include "freegame.h"
+#include "settings.h"
 
 #include "debug.h"
 
 
-FreeGame::FreeGame(const Dictionary &iDic): Game(iDic)
+FreeGame::FreeGame(const Dictionary &iDic)
+    : Game(iDic)
 {
 }
 
 
-FreeGame::~FreeGame()
-{
-}
-
-
-int FreeGame::setRackRandom(int p, bool iCheck, set_rack_mode mode)
-{
-    int res;
-    do
-    {
-        res = helperSetRackRandom(p, iCheck, mode);
-    } while (res == 2);
-    return res;
-}
-
-
 int FreeGame::play(const wstring &iCoord, const wstring &iWord)
 {
-    /* Perform all the validity checks, and fill a round */
+    // Perform all the validity checks, and try to fill a round
     Round round;
-
     int res = checkPlayedWord(iCoord, iWord, round);
-    if (res != 0)
+    if (res != 0 && Settings::Instance().getBool("freegame-reject-invalid"))
     {
         return res;
     }
 
-    /* Update the rack and the score of the current player */
-    m_players[m_currPlayer]->addPoints(round.getPoints());
-    m_players[m_currPlayer]->endTurn(round, m_history.getSize());
+    // If we reach this point, either the move is valid and we can use the
+    // "round" variable, or it is invalid but played nevertheless
+    if (res == 0)
+    {
+        Move move(round);
 
-    /* Everything is OK, we can play the word */
-    helperPlayRound(round);
+        // Update the rack and the score of the current player
+        m_players[m_currPlayer]->endTurn(move, m_history.getSize());
 
-    /* Next turn */
-    // XXX: Should it be done by the interface instead?
+        // Everything is OK, we can play the word
+        helperPlayMove(m_currPlayer, move);
+    }
+    else
+    {
+        Move move(iWord, iCoord);
+
+        // Record the invalid move of the player
+        m_players[m_currPlayer]->endTurn(move, m_history.getSize());
+
+        // Update the game
+        helperPlayMove(m_currPlayer, move);
+    }
+
+    // Next turn
     endTurn();
 
     return 0;
 }
 
 
-void FreeGame::freegameAI(int n)
+void FreeGame::playAI(unsigned int p)
 {
-    ASSERT(0 <= n && n < getNPlayers(), "Wrong player number");
-    ASSERT(!m_players[n]->isHuman(), "AI requested for a human player");
+    ASSERT(p < getNPlayers(), "Wrong player number");
+    ASSERT(!m_players[p]->isHuman(), "AI requested for a human player");
 
-    AIPlayer *player = static_cast<AIPlayer*>(m_players[n]);
+    AIPlayer *player = static_cast<AIPlayer*>(m_players[p]);
 
-    player->compute(*m_dic, m_board, m_history.getSize());
-    if (player->changesLetters())
+    player->compute(m_dic, m_board, m_history.beforeFirstRound());
+    const Move move = player->getMove();
+    if (move.getType() == Move::CHANGE_LETTERS ||
+        move.getType() == Move::PASS)
     {
-        helperPass(player->getChangedLetters(), n);
-        endTurn();
+        ASSERT(checkPass(move.getChangedLetters(), p) == 0, "AI tried to cheat!");
     }
-    else
-    {
-        const Round &round = player->getChosenRound();
-        /* Update the rack and the score of the current player */
-        player->addPoints(round.getPoints());
-        player->endTurn(round, m_history.getSize());
 
-        helperPlayRound(round);
-        endTurn();
-    }
+    // Update the rack and the score of the current player
+    player->endTurn(move, m_history.getSize());
+
+    helperPlayMove(p, move);
+    endTurn();
 }
 
 
@@ -109,19 +109,18 @@ int FreeGame::start()
 {
     ASSERT(getNPlayers(), "Cannot start a game without any player");
 
-    /* Set the initial racks of the players */
-    for (int i = 0; i < getNPlayers(); i++)
+    // Set the initial racks of the players
+    for (unsigned int i = 0; i < getNPlayers(); i++)
     {
-        setRackRandom(i, false, RACK_NEW);
+        helperSetRackRandom(i, false, RACK_NEW);
     }
 
-    // XXX
     m_currPlayer = 0;
 
-    /* If the first player is an AI, make it play now */
-    if (!m_players[0]->isHuman())
+    // If the first player is an AI, make it play now
+    if (!m_players[m_currPlayer]->isHuman())
     {
-        freegameAI(0);
+        playAI(m_currPlayer);
     }
 
     return 0;
@@ -130,21 +129,21 @@ int FreeGame::start()
 
 int FreeGame::endTurn()
 {
-    /* Complete the rack for the player that just played */
-    if (setRackRandom(m_currPlayer, false, RACK_NEW) == 1)
+    // Complete the rack for the player that just played
+    if (helperSetRackRandom(m_currPlayer, false, RACK_NEW) == 1)
     {
-        /* End of the game */
-        end();
+        // End of the game
+        endGame();
         return 1;
     }
 
-    /* Next player */
+    // Next player
     nextPlayer();
 
-    /* If this player is an AI, make it play now */
+    // If this player is an AI, make it play now
     if (!m_players[m_currPlayer]->isHuman())
     {
-        freegameAI(m_currPlayer);
+        playAI(m_currPlayer);
     }
 
     return 0;
@@ -152,7 +151,7 @@ int FreeGame::endTurn()
 
 
 // Adjust the scores of the players with the points of the remaining tiles
-void FreeGame::end()
+void FreeGame::endGame()
 {
     vector<Tile> tiles;
 
@@ -165,12 +164,12 @@ void FreeGame::end()
     // We currently handle case 1, and cannot handle case 3 until timers are
     // implemented.
     // For case 2, we need both to detect a blocked situation (not easy...) and
-    // to handle it in the end() method (very easy).
+    // to handle it in the endGame() method (very easy).
 
     /* Add the points of the remaining tiles to the score of the current
      * player (i.e. the first player with an empty rack), and remove them
      * from the score of the players who still have tiles */
-    for (int i = 0; i < getNPlayers(); i++)
+    for (unsigned int i = 0; i < getNPlayers(); i++)
     {
         if (i != m_currPlayer)
         {
@@ -184,73 +183,73 @@ void FreeGame::end()
         }
     }
 
-    /* Lock game */
+    // Lock game
     m_finished = true;
 }
 
 
-int FreeGame::pass(const wstring &iToChange, int n)
+int FreeGame::checkPass(const wstring &iToChange, unsigned int p) const
 {
+    ASSERT(p < getNPlayers(), "Wrong player number");
+
+    // Check that the game is not finished
     if (m_finished)
         return 3;
 
+    // Check that the letters are valid for the current dictionary
+    if (!m_dic.validateLetters(iToChange))
+        return 4;
+
+    // It is forbidden to change letters when the bag does not contain at
+    // least 7 letters (this is explicitly stated in the ODS). But it is
+    // still allowed to pass
+    Bag bag(m_dic);
+    realBag(bag);
+    if (bag.getNbTiles() < 7 && !iToChange.empty())
+    {
+        return 1;
+    }
+
+    // Check that the letters are all present in the player's rack
+    Player *player = m_players[p];
+    PlayedRack pld = player->getCurrentRack();
+    Rack rack;
+    pld.getRack(rack);
+    for (unsigned int i = 0; i < iToChange.size(); i++)
+    {
+        // Remove the letter from the rack
+        if (!rack.in(Tile(iToChange[i])))
+        {
+            return 2;
+        }
+        rack.remove(Tile(iToChange[i]));
+    }
+
     // According to the rules in the ODS, it is allowed to pass its turn (no
     // need to change letters for that).
     // TODO: However, if all the players pass their turn, the first one has to
     // play, or change at least one letter. To implement this behaviour, we
     // must also take care of blocked positions, where no one _can_ play (see
-    // also comment in the end() method).
+    // also comment in the endGame() method).
 
-    // Convert the string into tiles
-    vector<Tile> tilesVect;
-    for (unsigned int i = 0; i < iToChange.size(); i++)
-    {
-        Tile tile(towupper(iToChange[i]));
-        tilesVect.push_back(tile);
-    }
-
-    int res = helperPass(tilesVect, n);
-    if (res == 0)
-        endTurn();
-    return res;
+    return 0;
 }
 
 
-int FreeGame::helperPass(const vector<Tile> &iToChange, int n)
+int FreeGame::pass(const wstring &iToChange)
 {
-    ASSERT(0 <= n && n < getNPlayers(), "Wrong player number");
+    int res = checkPass(iToChange, m_currPlayer);
+    if (res != 0)
+        return res;
 
-    // It is forbidden to change letters when the bag does not contain at
-    // least 7 letters (this is explicitely stated in the ODS).
-    Bag bag;
-    realBag(bag);
-    if (bag.nTiles() < 7)
-    {
-        return 1;
-    }
+    Move move(iToChange);
+    // End the player's turn
+    m_players[m_currPlayer]->endTurn(move, m_history.getSize());
+    // Update the game
+    helperPlayMove(m_currPlayer, move);
 
-    Player *player = m_players[n];
-    PlayedRack pld = player->getCurrentRack();
-    Rack rack;
-    pld.getRack(rack);
-
-    for (unsigned int i = 0; i < iToChange.size(); i++)
-    {
-        /* Remove the letter from the rack */
-        if (!rack.in(iToChange[i]))
-        {
-            return 2;
-        }
-        rack.remove(iToChange[i]);
-    }
-
-    pld.reset();
-    pld.setOld(rack);
-
-    player->setCurrentRack(pld);
-
-    // FIXME: the letters to change should not be in the bag while generating
-    // the new rack!
+    // Next game turn
+    endTurn();
 
     return 0;
 }
