@@ -1,7 +1,8 @@
 /*****************************************************************************
- * Copyright (C) 1999-2005 Eliot
- * Authors: Antoine Fraboulet <antoine.fraboulet@free.fr>
- *          Olivier Teuliere  <ipkiss@via.ecp.fr>
+ * Eliot
+ * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
+ *          Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +25,7 @@
 #include <string>
 
 #include "game.h"
+#include "round.h"
 #include "results.h"
 
 using std::string;
@@ -34,6 +36,7 @@ using std::wstring;
  * This class handles the logic specific to a training game.
  * As its name indicates, it is not a game in the literal meaning of the word,
  * in particular because the rack can be set at will.
+ *
  * Note: No player should be added to this game, a human player is added
  * automatically (in the start() method)
  */
@@ -48,10 +51,13 @@ public:
      * Game handling
      *************************/
     virtual int start();
+
+    /// See description of Game::play()
     virtual int play(const wstring &iCoord, const wstring &iWord);
-    virtual int endTurn();
+
     void search();
-    int playResult(int);
+    const Results& getResults() const { return m_results; };
+    int playResult(unsigned int);
 
     int setRackRandom(bool, set_rack_mode);
     int setRackManual(bool iCheck, const wstring &iLetters);
@@ -59,7 +65,7 @@ public:
 
     /*************************
      * Override the default behaviour of these methods, because in training
-     * we only want a human player
+     * mode we only want a human player
      *************************/
     virtual void addHumanPlayer();
     virtual void addAIPlayer();
@@ -68,23 +74,25 @@ public:
      * Functions to access the current search results
      * The int parameter should be 0 <= int < getNResults
      *************************/
-    const Results& getResults() const { return m_results; };
 
-    /// Place a temporary word on the board for preview purpose
-    void testPlay(int);
-    /// Remove the temporary word(s)
+    /// Place a temporary word on the board for preview purposes
+    void testPlay(unsigned int);
+    /// Remove the temporary word
     void removeTestPlay();
     /// Get the temporary word
     wstring getTestPlayWord() const;
 
 private:
-    // Private constructor and destructor to force using the GameFactory class
+    /// Private constructor and destructor to force using the GameFactory class
     Training(const Dictionary &iDic);
-    virtual ~Training();
 
-    // Search results, with all the possible rounds
-    Round   m_testRound;
+    void endTurn();
+
+    /// Search results, with all the possible rounds
     Results m_results;
+
+    /// Round corresponding to the last test play (if any)
+    Round m_testRound;
 };
 
 #endif /* _TRAINING_H_ */
