@@ -24,10 +24,13 @@
  *  \date   2005
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <iostream>
 #include <fstream>
+#include <exception>
 
 using namespace std;
 
@@ -42,6 +45,7 @@ using namespace std;
 
 #include "dic.h"
 #include "game.h"
+#include "training.h"
 #include "game_factory.h"
 #include "player.h"
 
@@ -58,6 +62,7 @@ using namespace std;
 enum
 {
     Menu_Game_New                             = 1000,
+    Menu_Game_NewJoker,
     Menu_Game_Open,
     Menu_Game_Save,
     Menu_Game_Print,
@@ -100,9 +105,6 @@ enum
     Menu_ShowGame                             = (IDBASE + ID_Frame_Game),
     Menu_ShowResult                           = (IDBASE + ID_Frame_Result),
 
-    Menu_Quit_Apropos                         = 4000,
-    Menu_Quit_Confirm,
-
     Button_SetRack                            = 10000,
     Button_SetNew,
     Button_SetManual,
@@ -112,17 +114,19 @@ enum
 
     ListCtrl_ID                               = 11000,
     Rack_ID,
-    Status_ID,
+    Status_ID
 };
 
 BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     //
     EVT_MENU(Menu_Game_New,          MainFrame::OnMenuGameNew)
+    EVT_MENU(Menu_Game_NewJoker,     MainFrame::OnMenuGameNew)
     EVT_MENU(Menu_Game_Open,         MainFrame::OnMenuGameOpen)
     EVT_MENU(Menu_Game_Save,         MainFrame::OnMenuGameSave)
     EVT_MENU(Menu_Game_Print,        MainFrame::OnMenuGamePrint)
     EVT_MENU(Menu_Game_PrintPreview, MainFrame::OnMenuGamePrintPreview)
     EVT_MENU(Menu_Game_PrintPS,      MainFrame::OnMenuGamePrintPS)
+    EVT_MENU(wxID_EXIT,              MainFrame::OnMenuGameQuit)
     //
     EVT_MENU(Menu_Conf_Game_Dic,     MainFrame::OnMenuConfGameDic)
     EVT_MENU(Menu_Conf_Game_Search,  MainFrame::OnMenuConfGameSearch)
@@ -133,8 +137,7 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU_RANGE(Menu_Conf_Aspect_BoardColour_Lines, Menu_Conf_Aspect_BoardColour_Default, MainFrame::OnMenuConfAspectBoardColour)
     EVT_MENU_RANGE(Menu_ShowVerif, Menu_ShowResult, MainFrame::OnMenuShowFrame)
     //
-    EVT_MENU(Menu_Quit_Apropos, MainFrame::OnMenuQuitApropos)
-    EVT_MENU(Menu_Quit_Confirm, MainFrame::OnMenuQuitConfirm)
+    EVT_MENU(wxID_ABOUT, MainFrame::OnMenuHelpAbout)
     //
     EVT_BUTTON(Button_Play,     MainFrame::OnPlay)
     EVT_BUTTON(Button_SetRack,  MainFrame::OnSetRack)
@@ -152,35 +155,24 @@ END_EVENT_TABLE()
 // ******************************
 //
 // ******************************
-    
-MainFrame::MainFrame(wxPoint __UNUSED__ pos_, wxSize size_) : 
-           wxFrame((wxFrame *) NULL, -1, wxT(APPNAME), wxPoint(-1, -1),
-                   size_, wxDEFAULT_FRAME_STYLE, wxT(APPNAME)), m_dic(NULL), m_game(NULL)
+
+MainFrame::MainFrame(wxPoint __UNUSED__ pos_, wxSize size_)
+    : wxFrame((wxFrame *) NULL, -1, wxT(APPNAME), wxPoint(-1, -1),
+              size_, wxDEFAULT_FRAME_STYLE, wxT(APPNAME)),
+    m_dic(NULL), m_game(NULL)
 {
 #if defined(ENABLE_RESLIST_IN_MAIN)
     reslist = NULL;
 #endif
     statusbar = NULL;
-    for(int i=0 ; i < MAX_FRAME_ID; i++)
-      auxframes_ptr[i] = NULL;
-
-    wxString dicpath = config.getDicPath();
-    Dic_load(&m_dic, dicpath.mb_str());
-    if (m_dic == NULL)
-	{
-	    wxCommandEvent event;
-	    OnMenuConfGameDic(event);
-	}
-    m_game = GameFactory::Instance()->createTraining(m_dic);
-    if (m_game)
-	{
-	    m_game->start();
-	}
+    for (int i = 0 ; i < MAX_FRAME_ID; i++)
+        auxframes_ptr[i] = NULL;
 
     wxBoxSizer *listsizer = new wxBoxSizer(wxVERTICAL);
     rack = new wxTextCtrl(this, Rack_ID, wxU(""), wxPoint(-1, -1), wxSize(-1, -1), wxTE_PROCESS_ENTER);
-    listsizer->Add(rack    , 0 , wxEXPAND | wxALL, 1);
-    rack->SetToolTip(_("Tirage"));
+    listsizer->Add(rack, 0, wxEXPAND | wxALL, 1);
+    rack->SetToolTip(_("Rack"));
+    rack->Enable(false);
 #ifdef ENABLE_RESLIST_IN_MAIN
     reslist = new GfxResult(this,(MainFrame*)this,m_game);
     listsizer->Add(reslist, 1, wxEXPAND | wxLEFT | wxRIGHT, 1);
@@ -200,10 +192,15 @@ MainFrame::MainFrame(wxPoint __UNUSED__ pos_, wxSize size_) :
     b_play          = new wxButton(this, Button_Play,     _(" Play "));
 
     b_rackrandomset->SetToolTip(_("Random rack"));
-    b_rackrandomnew->SetToolTip(_("Random rack complement"));
-    b_search->SetToolTip(       _("Search from current rack"));
-    b_back->SetToolTip(         _("Go one turn back in the game"));
+    b_rackrandomset->Enable(false);
+    b_rackrandomnew->SetToolTip(_("Random complement of the rack"));
+    b_rackrandomnew->Enable(false);
+    b_search->SetToolTip(       _("Search with the current rack"));
+    b_search->Enable(false);
+    b_back->SetToolTip(         _("Go back one turn"));
+    b_back->Enable(false);
     b_play->SetToolTip(         _("Play the selected word"));
+    b_play->Enable(false);
 
     wxBoxSizer *buttonsizer = new wxBoxSizer(wxHORIZONTAL);
     buttonsizer->Add(b_rackrandomset, 1, wxEXPAND | wxTOP | wxBOTTOM | wxLEFT , 1);
@@ -224,6 +221,24 @@ MainFrame::MainFrame(wxPoint __UNUSED__ pos_, wxSize size_) :
     SetClientSize(size_);
     Move(config.getFramePos(wxT(APPNAME)));
 
+    wxString dicpath = config.getDicPath();
+    try
+    {
+        Dictionary *dic = new Dictionary(dicpath.mb_str().data());
+        m_dic = dic;
+        m_game = GameFactory::Instance()->createTraining(*m_dic);
+        if (m_game)
+        {
+            m_game->start();
+        }
+    }
+    catch (std::exception &e)
+    {
+        wxCommandEvent event;
+        // This will also start a new training game indirectly
+        OnMenuConfGameDic(event);
+    }
+
     InitFrames();
 }
 
@@ -237,15 +252,12 @@ MainFrame::~MainFrame()
     config.setFrameSize(wxT(APPNAME), GetClientSize());
 
     if (m_game != NULL)
-      {
-	GameFactory::Instance()->releaseGame(*m_game);
-	m_game = NULL;
-      }
+    {
+        GameFactory::Instance()->releaseGame(*m_game);
+        m_game = NULL;
+    }
 
-    if (m_dic)
-      {
-	Dic_destroy(m_dic);
-      }
+    delete m_dic;
 }
 
 // ******************************
@@ -257,84 +269,78 @@ MainFrame::InitMenu()
 {
     // menus
     wxMenu *menu_game = new wxMenu;
-    menu_game->Append(Menu_Game_New, _("Nouvelle"), _("Demarrer une nouvelle partie"));
-    menu_game->Append(Menu_Game_Open, _("Charger..."), _("Charger une partie"));
-    menu_game->Append(Menu_Game_Save, _("Sauver..."), _("Sauver cette partie"));
+    menu_game->Append(Menu_Game_New, _("&New game\tCtrl+N"), _("Start a new game"));
+    menu_game->Append(Menu_Game_NewJoker, _("New &joker game\tCtrl+J"), _("Start a new joker game"));
     menu_game->AppendSeparator();
-    menu_game->Append(Menu_Game_Print, _("Imprimer..."), _("Imprimer cette partie"));
-    menu_game->Append(Menu_Game_PrintPreview, _("Preimpression"), _("Preimpression de la partie"));
+    menu_game->Append(Menu_Game_Open, _("&Load...\tCtrl+L"), _("Load a game"));
+    menu_game->Append(Menu_Game_Save, _("&Save as...\tCtrl+S"), _("Save the current game"));
+    menu_game->AppendSeparator();
+    menu_game->Append(Menu_Game_Print, _("&Print...\tCtrl+P"), _("Print this game"));
+    menu_game->Append(Menu_Game_PrintPreview, _("Print pre&view..."), _("Print preview of the game"));
 #ifdef ENABLE_SAVE_POSTSCRIPT
-    menu_game->AppendSeparator();
-    menu_game->Append(Menu_Game_PrintPS, _("Imprimer du PostScript"), _("Imprimer dans un fichier PostScript"));
+    menu_game->Append(Menu_Game_PrintPS, _("Print in PostS&cript..."), _("Print in a PostScript file"));
 #endif
+    menu_game->AppendSeparator();
+    menu_game->Append(wxID_EXIT, _("&Quit\tCtrl+Q"), _("Quit Eliot"));
     //
     wxMenu *menu_conf_game = new wxMenu;
-    menu_conf_game->Append(Menu_Conf_Game_Dic, _("Dictionnaire"), _("Choix du dictionnaire"));
-    menu_conf_game->Append(Menu_Conf_Game_Search, _("Recherche"), _("Options de recherche"));
-    //
-    wxMenu *menu_tileback = new wxMenu;
-    menu_tileback->Append(Menu_Conf_Aspect_BoardColour_Letters     , _("Lettres jouees"), _("Lettres jouees sur la grille"));
-    menu_tileback->Append(Menu_Conf_Aspect_BoardColour_TestLetters , _("Lettres provisoires"), _("Lettres du mot a jouer"));
-    menu_tileback->AppendSeparator();
-    menu_tileback->Append(Menu_Conf_Aspect_BoardColour_TileBack    , _("Fonds lettres jouees"), _("Fonds des pions sur la grille"));
-    menu_tileback->Append(Menu_Conf_Aspect_BoardColour_TestTileBack, _("Fonds lettres provisoires"), _("Fonds des pions sur la grille"));
+    menu_conf_game->Append(Menu_Conf_Game_Dic, _("&Dictionary...\tCtrl+D"), _("Choose a dictionary"));
+    menu_conf_game->Append(Menu_Conf_Game_Search, _("&Search..."), _("Search options"));
     //
     wxMenu *menu_conf_board_colour = new wxMenu;
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Background, _("Fond"), _("Couleur du fond"));
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Lines, _("Lignes"), _("Couleur des lignes"));
-    menu_conf_board_colour->Append(Menu_Conf_Tile, _("Pions et lettres"), menu_tileback, _("Pions et lettres"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Background, _("&Background..."), _("Background color"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Lines, _("L&ines..."), _("Color of the lines"));
     menu_conf_board_colour->AppendSeparator();
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Letters, _("Lettres jouees"), _("Lettres jouees sur la grille"));
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_TestLetters, _("Lettres provisoires"), _("Lettres du mot a jouer"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Letters, _("&Played letters..."), _("Color of the letters played on the board"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_TestLetters, _("&Temporary letters..."), _("Color of the letters of the temporary word"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_TileBack    , _("B&ackground of played letters..."), _("Background color of the letters played on the board"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_TestTileBack, _("Ba&ckground of temporary letters..."), _("Background color of the temporary letters on the board"));
     menu_conf_board_colour->AppendSeparator();
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Wx2, _("Mot compte double"), _("Mot compte double"));
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Wx3, _("Mot compte triple"), _("Mot compte triple"));
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Lx2, _("Lettre compte double"), _("Lettre compte double"));
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Lx3, _("Lettre compte triple"), _("Lettre compte triple"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Wx2, _("Double &letter..."), _("Color of the \"double letter\" squares"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Wx3, _("Triple l&etter..."), _("Color of the \"triple letter\" squares"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Lx2, _("Double &word..."), _("Color of the \"double word\" squares"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Lx3, _("Triple w&ord..."), _("Color of the \"triple word\" squares"));
     menu_conf_board_colour->AppendSeparator();
-    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Default, _("Couleurs d'origine"), _("Retrouver les couleurs d'origine"));
+    menu_conf_board_colour->Append(Menu_Conf_Aspect_BoardColour_Default, _("&Default colors"), _("Restore the default colors"));
     //
     wxMenu *menu_conf_board_font = new wxMenu;
-    menu_conf_board_font->Append(Menu_Conf_Aspect_Font_Search, _("Search font"), _("Search list font"));
-    menu_conf_board_font->Append(Menu_Conf_Aspect_Font_Board,  _("Board font"), _("Game board font"));
+    menu_conf_board_font->Append(Menu_Conf_Aspect_Font_Search, _("&Search letters..."), _("Font for the search"));
     //
     wxMenu *menu_conf = new wxMenu;
-    menu_conf->Append(Menu_Conf_Game, _("Jeu"), menu_conf_game, _("Configuration du jeu"));
-    menu_conf->Append(Menu_Conf_Aspect_Font, _("Fonte des lettres"), menu_conf_board_font, _("Modification des fontes"));
-    menu_conf->Append(Menu_Conf_Aspect_BoardColour, _("Couleurs de la grille"), menu_conf_board_colour, _("Modification des couleurs"));
-    menu_conf->Append(Menu_Conf_Print, _("Impression"), _("Dimensions de la partie"));
+    menu_conf->Append(Menu_Conf_Game, _("&Game"), menu_conf_game, _("Configuration of the game"));
+    menu_conf->Append(Menu_Conf_Aspect_Font, _("&Fonts"), menu_conf_board_font, _("Configuration of the fonts"));
+    menu_conf->Append(Menu_Conf_Aspect_BoardColour, _("&Colors"), menu_conf_board_colour, _("Configuration of the colors"));
+    menu_conf->Append(Menu_Conf_Print, _("&Printing..."), _("Configuration of the printing parameters"));
     //
     wxMenu *menu_frame = new wxMenu;
-    menu_frame->Append(Menu_ShowBoard, _("Board"), _("Game board"));
-    menu_frame->Append(Menu_ShowVerif, _("Verify"), _("Verify a word in the dictionary"));
-    menu_frame->Append(Menu_ShowSearch, _("Search"), _("Search best results"));
+    menu_frame->AppendCheckItem(Menu_ShowBoard, _("&Board"), _("Game board"));
+    menu_frame->AppendCheckItem(Menu_ShowBag, _("Ba&g\tCtrl+B"), _("Remaining letters in the bag"));
+    menu_frame->AppendCheckItem(Menu_ShowVerif, _("&Check"), _("Check a word in the dictionary"));
+    menu_frame->AppendCheckItem(Menu_ShowSearch, _("&Search"), _("Search in the dictionary"));
     menu_frame->AppendSeparator();
-    menu_frame->Append(Menu_ShowPlus1, _("Rack + 1"), _("Lettres du tirage plus une"));
-    menu_frame->Append(Menu_ShowRacc, _("Raccords"), _("Raccords sur un mot de la recherche"));
-    menu_frame->Append(Menu_ShowBenj, _("Benjamins"), _("Benjamins sur un mot de la recherche"));
+    menu_frame->AppendCheckItem(Menu_ShowPlus1, _("&Rack + 1"), _("Letters of the rack plus one"));
+    menu_frame->AppendCheckItem(Menu_ShowRacc, _("&Extensions"), _("One-letter extensions on a word of the search"));
+    menu_frame->AppendCheckItem(Menu_ShowBenj, _("&Benjamins"), _("Benjamins (3-letter extensions) on a word of the search"));
     menu_frame->AppendSeparator();
-    menu_frame->Append(Menu_ShowBag, _("Bag"), _("Lettres restantes dans le sac"));
-    menu_frame->AppendSeparator();
-    menu_frame->Append(Menu_ShowGame, _("Game"), _("Game"));
+    menu_frame->AppendCheckItem(Menu_ShowGame, _("Game &history\tCtrl+H"), _("Game history"));
 #ifndef ENABLE_RESLIST_IN_MAIN
-    menu_frame->Append(Menu_ShowResult, _("Results"), _("Results"));
+    menu_frame->Append(Menu_ShowResult, _("R&esults"), _("Results"));
 #endif
     //
-    wxMenu *menu_quit = new wxMenu;
-    menu_quit->Append(Menu_Quit_Apropos, _("A propos..."), _("A propos d'Eliot"));
-    menu_quit->Append(Menu_Quit_Confirm, _("Quit"), _("Quit"));
+    wxMenu *menu_help = new wxMenu;
+    menu_help->Append(wxID_ABOUT, _("&About..."), _("About Eliot"));
     //
     wxMenuBar *menu_bar = new wxMenuBar;
-    menu_bar->Append(menu_game,  _("Game"));
-    menu_bar->Append(menu_conf,  _("Configuration"));
-    menu_bar->Append(menu_frame, _("Windows"));
-    menu_bar->Append(menu_quit,  _("Eliot"));
+    menu_bar->Append(menu_game, _("&Game"));
+    menu_bar->Append(menu_conf, _("&Settings"));
+    menu_bar->Append(menu_frame, _("&Windows"));
+    menu_bar->Append(menu_help, _("&Help"));
 
     SetMenuBar(menu_bar);
 }
 
 // *******************
-// 
+//
 // *******************
 
 void
@@ -348,30 +354,43 @@ MainFrame::OnCloseWindow(wxCloseEvent&)
 // *******************
 
 void
-MainFrame::OnMenuGameNew(wxCommandEvent&)
+MainFrame::OnMenuGameNew(wxCommandEvent& event)
 {
     if (m_dic == NULL)
-      {
-        wxMessageBox(_("Il n'y a pas de dictionnaire selectionne"), _("Eliot: erreur"),
+    {
+        wxMessageBox(_("No dictionary selected"), _("Eliot: error"),
                      wxICON_INFORMATION | wxOK);
         return;
-      }
+    }
 
     //    TODO("selection du type de partie dans OnMenuGameNew\n");
 
     if (m_game != NULL)
-      {
-	GameFactory::Instance()->releaseGame(*m_game);
-	m_game = NULL;
-      }
+    {
+        GameFactory::Instance()->releaseGame(*m_game);
+        m_game = NULL;
+    }
+
+    m_game = GameFactory::Instance()->createTraining(*m_dic);
+    // Joker game?
+    if (event.GetId() == Menu_Game_NewJoker)
+        m_game->setVariant(Game::kJOKER);
 
-    m_game = GameFactory::Instance()->createTraining(m_dic);
     m_game->start();
-    rack->SetValue(wxU(""));
+    rack->SetValue(wxT(""));
     InitFrames();
 #ifdef ENABLE_RESLIST_IN_MAIN
     reslist->SetGame(m_game);
 #endif
+    // Re-enable the main buttons
+    b_rackrandomset->Enable(true);
+    b_rackrandomnew->Enable(true);
+    b_search->Enable(true);
+    b_back->Enable(true);
+    b_play->Enable(true);
+    rack->Enable(true);
+
+
     UpdateStatusBar();
     UpdateFrames(AuxFrame::FORCE_REFRESH);
 }
@@ -383,63 +402,58 @@ MainFrame::OnMenuGameNew(wxCommandEvent&)
 void
 MainFrame::OnMenuGameOpen(wxCommandEvent&)
 {
-    wxString txt;
-    wxFileDialog dialog(this, _("Ouvrir une partie"), wxT(""), wxT(""), wxT("*"), wxOPEN);
+    wxFileDialog dialog(this, _("Load a game"), wxT(""), wxT(""), wxT("*"), wxOPEN);
     if (m_dic == NULL)
     {
-        wxMessageBox(_("Il n'y a pas de dictionnaire selectionne"), _("Eliot: erreur"),
+        wxMessageBox(_("No dictionary selected"), _("Eliot: error"),
                      wxICON_INFORMATION | wxOK);
         return;
     }
     if (dialog.ShowModal() != wxID_OK)
-	{
-	    return;
-	}
+    {
+        return;
+    }
 
     if (m_game != NULL)
-	{
-	    GameFactory::Instance()->releaseGame(*m_game);
-	    m_game = NULL;
-	}
+    {
+        GameFactory::Instance()->releaseGame(*m_game);
+        m_game = NULL;
+    }
 
     FILE* fin;
 
     if ((fin = fopen(dialog.GetPath().mb_str(), "rb")) == NULL)
-        {
-            txt << _("Impossible d'ouvrir") << dialog.GetPath();
-            wxMessageDialog msg(this, txt, _("Ouverture d'une partie"));
-            msg.ShowModal();
-            return ;
-        }
+    {
+        wxString txt;
+        txt << _("Cannot open ") << dialog.GetPath();
+        wxMessageDialog msg(this, txt, _("Load a game"));
+        msg.ShowModal();
+        return ;
+    }
 
-    m_game = Game::load(fin, m_dic);
+    m_game = Game::load(fin, *m_dic);
     fclose(fin);
 
     if (m_game == NULL)
-        {
-            wxMessageDialog msg(this,
-				_("Erreur pendant la lecture de la partie"),
-				_("chargement de partie"));
-            msg.ShowModal();
-            return;
-        }
+    {
+        wxMessageDialog msg(this,
+                            _("Error while loading the game"),
+                            _("Invalid game"));
+        msg.ShowModal();
+        return;
+    }
 
     if (m_game->getHistory().getSize() == 0)
-	{
-            wxMessageDialog msg(this,
-				_("Erreur pendant la lecture de la partie"),
-				_("La partie est vide"));
-            msg.ShowModal();
-            return;
-	}
-
-    std::wstring r;
-
-    if (m_game->getHistory().getSize() >= 0)
     {
-        r = m_game->getCurrentPlayer().getCurrentRack().toString();
+        wxMessageDialog msg(this,
+                            _("Error while loading the game"),
+                            _("The game is empty"));
+        msg.ShowModal();
+        return;
     }
 
+    std::wstring r = m_game->getCurrentPlayer().getCurrentRack().toString();
+
     rack->SetValue(wxU(r.c_str()));
     // update gfxboard and all frames
     InitFrames();
@@ -458,15 +472,15 @@ MainFrame::OnMenuGameOpen(wxCommandEvent&)
 void
 MainFrame::OnMenuGameSave(wxCommandEvent& WXUNUSED(event))
 {
-    wxFileDialog dialog(this, _("Sauver une partie"), wxT(""), wxT(""), wxT("*"), wxSAVE|wxOVERWRITE_PROMPT);
+    wxFileDialog dialog(this, _("Save the game"), wxT(""), wxT(""), wxT("*"), wxSAVE|wxOVERWRITE_PROMPT);
     if (dialog.ShowModal() == wxID_OK)
     {
         ofstream fout(dialog.GetPath().mb_str());
         if (fout.rdstate() == ios::failbit)
         {
             wxString txt;
-            txt << _("Impossible de creer ") << dialog.GetPath();
-            wxMessageDialog msg(this, txt, _("Sauvegarde de la partie"));
+            txt << _("Cannot create ") << dialog.GetPath();
+            wxMessageDialog msg(this, txt, _("Save the game"));
             msg.ShowModal();
             return ;
         }
@@ -485,7 +499,7 @@ MainFrame::OnMenuGamePrint(wxCommandEvent& WXUNUSED(event))
     // TODO: gray out the menu instead...
     if (m_game == NULL)
     {
-        wxMessageBox(_("Pas de partie en cours"), _("Eliot: erreur"),
+        wxMessageBox(_("No on going game"), _("Eliot: error"),
                      wxICON_INFORMATION | wxOK);
         return;
     }
@@ -493,7 +507,10 @@ MainFrame::OnMenuGamePrint(wxCommandEvent& WXUNUSED(event))
     wxPrinter printer(&printDialogData);
     GamePrintout printout(*m_game);
     if (!printer.Print(this, &printout, TRUE))
-        wxMessageBox(_("Impression non effectuee."));
+    {
+        wxMessageBox(_("Printing not done"), _("Printing"),
+                     wxOK | wxICON_ERROR);
+    }
 }
 
 void
@@ -502,7 +519,7 @@ MainFrame::OnMenuGamePrintPreview(wxCommandEvent& WXUNUSED(event))
     // TODO: gray out the menu instead...
     if (m_game == NULL)
     {
-        wxMessageBox(_("Pas de partie en cours"), _("Eliot: erreur"),
+        wxMessageBox(_("No on going game"), _("Eliot: error"),
                      wxICON_INFORMATION | wxOK);
         return;
     }
@@ -514,12 +531,12 @@ MainFrame::OnMenuGamePrintPreview(wxCommandEvent& WXUNUSED(event))
     if (!preview->Ok())
     {
         delete preview;
-        msg << _("Probleme de previsualisation.\n")
-            << _("Il se peut que l'imprimante par defaut soit mal initialisee");
-        wxMessageBox(msg, _("Impression (previsualisation)"), wxOK);
+        msg << _("Print preview problem.\n")
+            << _("The printer may not be correctly initialized");
+        wxMessageBox(msg, _("Print preview"), wxOK);
         return;
     }
-    wxPreviewFrame *frame = new wxPreviewFrame(preview, this, _("Impression"),
+    wxPreviewFrame *frame = new wxPreviewFrame(preview, this, _("Printing"),
                                                wxPoint(-1, -1), wxSize(600, 550));
     frame->Centre(wxBOTH);
     frame->Initialize();
@@ -533,12 +550,11 @@ MainFrame::OnMenuGamePrintPS(wxCommandEvent& WXUNUSED(event))
     // TODO: gray out the menu instead...
     if (m_game == NULL)
     {
-        wxMessageBox(_("Pas de partie en cours"), _("Eliot: erreur"),
+        wxMessageBox(_("No on going game"), _("Eliot: error"),
                      wxICON_INFORMATION | wxOK);
         return;
     }
-    wxString txt;
-    wxFileDialog dialog(this, _("Imprimer dans un fichier PostScript"), wxT(""), wxT(""), wxT("*.ps"), wxSAVE|wxOVERWRITE_PROMPT);
+    wxFileDialog dialog(this, _("Print to a PostScript file"), wxT(""), wxT(""), wxT("*.ps"), wxSAVE|wxOVERWRITE_PROMPT);
     if (dialog.ShowModal() == wxID_OK)
     {
         wxPrintData printdataPS;
@@ -556,68 +572,54 @@ MainFrame::OnMenuGamePrintPS(wxCommandEvent& WXUNUSED(event))
             GamePrintout printout(*m_game);
             if (!printer.Print(this, &printout, FALSE))
             {
-                wxMessageBox(_("Impression non effectuee."));
-            }
-            else
-            {
-                wxString msg;
-                msg << _("Dessin effectue dans ") << dialog.GetPath() << _("\n");
-                wxMessageBox(msg, _("Sauvegarde PostScript"), wxOK);
+                wxMessageBox(_("Printing not done"),
+                             _("PostScript printing"), wxOK | wxICON_ERROR);
             }
         }
         else
         {
-            wxString msg;
-            msg << _("impossible d'initialiser le traitement PostScript.\n");
-            wxMessageBox(msg, _("Sauvegarde PostScript"), wxOK);
+            wxMessageBox(_("Cannot initialize PostScript printer"),
+                         _("PostScript printing"), wxOK | wxICON_ERROR);
         }
     }
 #endif
 }
 
 
+void
+MainFrame::OnMenuGameQuit(wxCommandEvent& WXUNUSED(event))
+{
+    Close(TRUE);
+}
+
+
+
 // *******************
-// Dictionnary Loading
+// Dictionary Loading
 // *******************
 
 void
 MainFrame::OnMenuConfGameDic(wxCommandEvent& WXUNUSED(event))
 {
-    wxString txt, msg, dicpath;
-    wxFileDialog dialog(this, _("Choisir un dictionnaire"), wxT(""), wxT("*.dawg"), wxT("*.dawg"), wxOPEN);
+    wxFileDialog dialog(this, _("Choose a dictionary"), wxT(""), wxT("*.dawg"), wxT("*.dawg"), wxOPEN);
     if (dialog.ShowModal() == wxID_OK)
     {
         wxString dicpath = dialog.GetPath();
-        Dictionary dic;
-        int res = Dic_load(&dic, dicpath.mb_str());
-        if (res == 0)
+        try
         {
-            if (m_dic)
-	      {
-                Dic_destroy(m_dic);
-	      }
-
+            Dictionary *dic = new Dictionary(dicpath.mb_str().data());
+            delete m_dic;
             m_dic = dic;
             config.setDicPath(dialog.GetPath(), ::wxFileNameFromPath(dialog.GetPath()));
+            wxCommandEvent event;
+            OnMenuGameNew(event);
         }
-        else
+        catch (std::exception &e)
         {
-            switch (res)
-            {
-                case 0: /* cas normal */ break;
-                case 1:  msg << _("chargement: probleme d'ouverture de ") << dicpath << _("\n"); break;
-                case 2:  msg << _("chargement: mauvais en-tete de dictionnaire\n"); break;
-                case 3:  msg << _("chargement: probleme 3 d'allocation memoire\n"); break;
-                case 4:  msg << _("chargement: probleme 4 d'allocation memoire\n"); break;
-                case 5:  msg << _("chargement: probleme de lecture des arcs du dictionnaire\n"); break;
-                default: msg << _("chargement: probleme non-repertorie\n"); break;
-            }
-            wxMessageDialog dlg(NULL, msg, wxT(APPNAME));
+            wxMessageDialog dlg(NULL, wxU(e.what()), wxT(APPNAME));
             dlg.ShowModal();
         }
     }
-    UpdateStatusBar();
-    UpdateFrames();
 }
 
 // ****************
@@ -700,8 +702,8 @@ MainFrame::OnMenuConfAspectBoardColour(wxCommandEvent& event)
         case Menu_Conf_Aspect_BoardColour_Background:   attr = wxString(BCOLOURBACKGROUND); break;
         case Menu_Conf_Aspect_BoardColour_Letters:      attr = wxString(BCOLOURLETTERS); break;
         case Menu_Conf_Aspect_BoardColour_TestLetters:  attr = wxString(BCOLOURTSTLETTERS); break;
-	case Menu_Conf_Aspect_BoardColour_TileBack:     attr = wxString(BTILEBACKGROUND); break;
-	case Menu_Conf_Aspect_BoardColour_TestTileBack: attr = wxString(BTSTTILEBACKGROUND); break;
+        case Menu_Conf_Aspect_BoardColour_TileBack:     attr = wxString(BTILEBACKGROUND); break;
+        case Menu_Conf_Aspect_BoardColour_TestTileBack: attr = wxString(BTSTTILEBACKGROUND); break;
         case Menu_Conf_Aspect_BoardColour_Default:      attr = wxU("Default"); break;
         default: INCOMPLETE; break;
     }
@@ -715,30 +717,22 @@ MainFrame::OnMenuConfAspectBoardColour(wxCommandEvent& event)
 }
 
 //**************************************************************************************
-//   MENU QUIT
+//   MENU HELP
 //**************************************************************************************
 
 void
-MainFrame::OnMenuQuitApropos(wxCommandEvent& WXUNUSED(event))
+MainFrame::OnMenuHelpAbout(wxCommandEvent& WXUNUSED(event))
 {
     wxString msg;
-    // XXX:    msg << wxT("Eliot\n Antoine Fraboulet 1999-2004\n\n");
-    msg << wxT("Eliot\nCopyright Antoine Fraboulet 1999-2006\n\n");
-    msg << wxT("This program is free software; you can redistribute it and/or modify\n");
-    msg << wxT("it under the terms of the GNU General Public License as published by\n");
-    msg << wxT("the Free Software Foundation; either version 2 of the License, or\n");
-    msg << wxT("(at your option) any later version.\n\n");
-    msg << wxT("Version ") << wxT(VERSION) << wxT("\n");
-    wxMessageBox(msg, _("A propos d'Eliot"), wxICON_INFORMATION | wxOK);
+    msg.Printf(wxT("Eliot %s\n\n"), wxT(VERSION));
+    msg << wxT("Copyright (C) 1999-2007 - Antoine Fraboulet & Olivier Teuliere\n\n");
+    msg << _("This program is free software; you can redistribute it and/or modify " \
+             "it under the terms of the GNU General Public License as published by " \
+             "the Free Software Foundation; either version 2 of the License, or " \
+             "(at your option) any later version.");
+    wxMessageBox(msg, _("About Eliot"), wxICON_INFORMATION | wxOK);
 }
 
-void
-MainFrame::OnMenuQuitConfirm(wxCommandEvent& WXUNUSED(event))
-{
-    Close(TRUE);
-}
-
-
 //**************************************************************************************
 // BUTTONS
 //**************************************************************************************
@@ -751,22 +745,22 @@ MainFrame::OnSetRack(wxCommandEvent& event)
     Game::set_rack_mode mode = Game::RACK_NEW;
     debug("OnSetRack ");
     switch ((id = event.GetId()))
-	{
+    {
         case Button_SetRack:
-	    mode = Game::RACK_ALL;
-	    debug("PlayedRack::RACK_ALL\n");
-	    break;
+            mode = Game::RACK_ALL;
+            debug("PlayedRack::RACK_ALL\n");
+            break;
         case Button_SetNew:
-	    mode = Game::RACK_NEW;
-	    debug("PlayedRack::RACK_NEW\n");
-	    break;
+            mode = Game::RACK_NEW;
+            debug("PlayedRack::RACK_NEW\n");
+            break;
         case Button_SetManual:
-	    mode = Game::RACK_MANUAL;
-	    debug("PlayedRack::RACK_MANUAL\n");
-	    break;
+            mode = Game::RACK_MANUAL;
+            debug("PlayedRack::RACK_MANUAL\n");
+            break;
         default:
-	    return;
-	}
+            return;
+    }
     SetRack(mode);
 }
 
@@ -793,16 +787,16 @@ MainFrame::OnPlay(wxCommandEvent& event)
     int dir = 1;
     int id = event.GetId();
     switch (id)
-	{
+    {
         case Button_Play:
-	    dir = 1;
+            dir = 1;
             break;
         case Button_PlayBack:
-	    dir = -1;
+            dir = -1;
             break;
         default:
             break;
-	}
+    }
     Play(dir);
 }
 
@@ -815,49 +809,59 @@ MainFrame::InitFrames()
 {
     debug("InitFrames start : \n");
     if (m_game == NULL)
-	{
-	    debug("m_game == NULL\n");
-	    return;
-	}
-    
-    for(int i=0 ; i < MAX_FRAME_ID; i++)
-	{
-	    if (auxframes_ptr[i] != NULL)
-		{
-		    debug("   delete frame %d\n",i);
-		    delete auxframes_ptr[i];
-		}
-	}
-    
-    auxframes_ptr[ ID_Frame_Verif  ] = new VerifFrame (this, m_game->getDic());
+    {
+        debug("m_game == NULL\n");
+        return;
+    }
+
+    for (int i = 0 ; i < MAX_FRAME_ID; i++)
+    {
+        debug("   delete frame %d\n",i);
+        delete auxframes_ptr[i];
+    }
+
+    auxframes_ptr[ID_Frame_Verif]  = new VerifFrame (this, m_game->getDic());
     debug("0 : Verif\n");
-    auxframes_ptr[ ID_Frame_Search ] = new SearchFrame(this, m_game->getDic());
+    auxframes_ptr[ID_Frame_Search] = new SearchFrame(this, m_game->getDic());
     debug("1 : Search\n");
-    auxframes_ptr[ ID_Frame_Plus1  ] = new Plus1Frame (this, m_game);
+    auxframes_ptr[ID_Frame_Plus1]  = new Plus1Frame (this, m_game);
     debug("2 : Plus1\n");
-    auxframes_ptr[ ID_Frame_Racc   ] = new RaccFrame  (this, m_game);
+    auxframes_ptr[ID_Frame_Racc]   = new RaccFrame  (this, m_game);
     debug("3 : Racc\n");
-    auxframes_ptr[ ID_Frame_Benj   ] = new BenjFrame  (this, m_game);
+    auxframes_ptr[ID_Frame_Benj]   = new BenjFrame  (this, m_game);
     debug("4 : Benj\n");
-    auxframes_ptr[ ID_Frame_Bag    ] = new BagFrame   (this, *m_game);
+    auxframes_ptr[ID_Frame_Bag]    = new BagFrame   (this, *m_game);
     debug("5 : Bag\n");
-    auxframes_ptr[ ID_Frame_Board  ] = new BoardFrame (this, *m_game);
+    auxframes_ptr[ID_Frame_Board]  = new BoardFrame (this, *m_game);
     debug("6 : Board\n");
-    auxframes_ptr[ ID_Frame_Game   ] = new GameFrame  (this, *m_game);
+    auxframes_ptr[ID_Frame_Game]   = new GameFrame  (this, *m_game);
     debug("7 : Game\n");
 #ifndef ENABLE_RESLIST_IN_MAIN
-    auxframes_ptr[ ID_Frame_Result ] = new ResultFrame(this, m_game);
+    auxframes_ptr[ID_Frame_Result] = new ResultFrame(this, m_game);
     debug("8 : Result\n");
 #endif
-    
+
     for (int i = MIN_FRAME_ID; i < MAX_FRAME_ID; i++)
     {
-	if (auxframes_ptr[i] != NULL)
-	{
-	    auxframes_ptr[i]->Reload();
-	    debug("reload %d\n",i);
-	}
+        if (auxframes_ptr[i] != NULL)
+        {
+            auxframes_ptr[i]->Reload();
+            debug("reload %d\n",i);
+        }
     }
+
+    // Check the corresponding menu item if the window is visible
+    GetMenuBar()->Check(Menu_ShowVerif, auxframes_ptr[ID_Frame_Verif]->IsShown());
+    GetMenuBar()->Check(Menu_ShowSearch, auxframes_ptr[ID_Frame_Search]->IsShown());
+    GetMenuBar()->Check(Menu_ShowPlus1, auxframes_ptr[ID_Frame_Plus1]->IsShown());
+    GetMenuBar()->Check(Menu_ShowRacc, auxframes_ptr[ID_Frame_Racc]->IsShown());
+    GetMenuBar()->Check(Menu_ShowBenj, auxframes_ptr[ID_Frame_Benj]->IsShown());
+    GetMenuBar()->Check(Menu_ShowBag, auxframes_ptr[ID_Frame_Bag]->IsShown());
+    GetMenuBar()->Check(Menu_ShowBoard, auxframes_ptr[ID_Frame_Board]->IsShown());
+    GetMenuBar()->Check(Menu_ShowGame, auxframes_ptr[ID_Frame_Game]->IsShown());
+#ifndef ENABLE_RESLIST_IN_MAIN
+    GetMenuBar()->Check(Menu_ShowResult, auxframes_ptr[ID_Frame_Result]->IsShown());
+#endif
     debug("InitFrames end ok.\n");
 }
 
@@ -866,21 +870,21 @@ MainFrame::OnMenuShowFrame(wxCommandEvent& event)
 {
     int id;
     id = event.GetId();
-    
+
     if (!GetMenuBar()->FindItem(id))
         return;
     id -= IDBASE;
-    
+
     if ((id < 0) || (id >= MAX_FRAME_ID))
     {
-	INCOMPLETE;
-	return;
+        INCOMPLETE;
+        return;
     }
-    
+
     if (auxframes_ptr[id] == NULL)
     {
-	debug("ShowFrame: auxframes_ptr[%d] == NULL\n", id);
-	return;
+        debug("ShowFrame: auxframes_ptr[%d] == NULL\n", id);
+        return;
     }
     auxframes_ptr[id]->SwitchDisplay();
     debug("ShowFrame: SwitchDisplay frame %d\n",id);
@@ -895,15 +899,15 @@ MainFrame::UpdateFrames(AuxFrame::refresh_t force)
 {
     for (int id = 0; id < MAX_FRAME_ID; id++)
     {
-	if (auxframes_ptr[id])
-	    {
-		auxframes_ptr[id]->Refresh(force);
-	    }
+        if (auxframes_ptr[id])
+        {
+            auxframes_ptr[id]->Refresh(force);
+        }
     }
 #ifdef ENABLE_RESLIST_IN_MAIN
     if (reslist)
     {
-	reslist->Refresh();
+        reslist->Refresh();
     }
 #endif
 }
@@ -914,16 +918,16 @@ MainFrame::UpdateStatusBar()
     wxString text;
     if (statusbar)
     {
-	text = config.getDicName() + wxT(" ") + config.getTileName();
-	statusbar->SetStatusText(text, 0);
-	
-	if (m_game)
-	{
-	    text = wxT("");
-	    text << _("coup:") << (m_game->getHistory().getSize() + 1) << wxT(" ");
-	    text << _("points:") << (m_game->getCurrentPlayer().getPoints());
-	    statusbar->SetStatusText(text, 1);
-	}
+        text = config.getDicName() + wxT(" ") + config.getTileName();
+        statusbar->SetStatusText(text, 0);
+
+        if (m_game)
+        {
+            text = wxT("");
+            text << _("turn:") << wxT(" ") << (m_game->getHistory().getSize() + 1) << wxT(" ");
+            text << _("points:") << wxT(" ") << (m_game->getCurrentPlayer().getPoints());
+            statusbar->SetStatusText(text, 1);
+        }
     }
 }
 
@@ -940,36 +944,37 @@ MainFrame::UpdateStatusBar()
 void
 MainFrame::SetRack(Game::set_rack_mode mode, wxString srack)
 {
-    int res = 0;
     wxString msg;
     bool check = config.getRackChecking();
 
+    if (m_game == NULL)
+    {
+        return;
+    }
     static_cast<Training*>(m_game)->removeTestPlay();
-    std::wstring str = srack.c_str();
-    res = static_cast<Training*>(m_game)->setRack(mode, check, str);
+    int res = static_cast<Training*>(m_game)->setRack(mode, check, srack.c_str());
 
     switch (res)
-	{
+    {
         case 0x00: /* ok */
-	    debug("SetRack Ok :: ");
-	    break;
+            debug("SetRack Ok :: ");
+            break;
         case 0x01:
-            msg = _("Le sac ne contient pas assez de lettres\npour assurer le tirage.");
-            wxMessageBox(msg, _("Correction du tirage"), wxICON_INFORMATION | wxOK);
+            msg = _("The bag doesn't contain enough letters\nfor a new rack.");
+            wxMessageBox(msg, _("Rack validation"), wxICON_ERROR | wxOK);
             return;
         case 0x02:
-            msg = _("Le tirage doit contenir au moins 2 consonnes et 2 voyelles.\n");
-            wxMessageBox(msg, _("Correction du tirage"), wxICON_INFORMATION | wxOK);
+            msg = _("The rack must contain at least 2 consonants and 2 vowels.");
+            wxMessageBox(msg, _("Rack validation"), wxICON_ERROR | wxOK);
             return;
         case 0x03:
-	    msg  = _("Le tirage doit contenir au moins 2 consonnes et 2 voyelles\n");
-	    msg += _("mais le sac ne contient plus assez de lettres.\n\n");
-	    wxMessageBox(msg, _("Correction du tirage"), wxICON_INFORMATION | wxOK);
+            msg = _("The rack contains invalid letters for the current dictionary");
+            wxMessageBox(msg, _("Rack validation"), wxICON_ERROR | wxOK);
             break;
-	default:
-	    statusbar->SetStatusText(_("Le tirage a ete modifie manuellement"), 0);
-	    break;
-	}
+        default:
+            statusbar->SetStatusText(_("The rack has been modified manually"), 0);
+            break;
+    }
 
     std::wstring r = m_game->getCurrentPlayer().getCurrentRack().toString();
     debug("MainFrame::SetRack : setvalue %ls\n",r.c_str());
@@ -981,7 +986,7 @@ MainFrame::SetRack(Game::set_rack_mode mode, wxString srack)
 void
 MainFrame::Search()
 {
-    if (m_game == NULL || m_game->getDic() == NULL)
+    if (m_game == NULL)
     {
         return;
     }
@@ -990,9 +995,9 @@ MainFrame::Search()
     reslist->Search();
 #else
     if (auxframes_ptr[ ID_Frame_Result ])
-	{
-	    ((ResultFrame*)(auxframes_ptr[ ID_Frame_Result ]))->Search();
-	}
+    {
+        ((ResultFrame*)(auxframes_ptr[ ID_Frame_Result ]))->Search();
+    }
 #endif
     UpdateFrames();
     UpdateStatusBar();
@@ -1007,24 +1012,24 @@ MainFrame::Play(int n)
     }
     static_cast<Training*>(m_game)->removeTestPlay();
     if (n < 0)
-	{
-	    debug("MainFrame::Play back %d\n",n);
-	    m_game->back(- n);
-	}
+    {
+        debug("MainFrame::Play back %d\n",n);
+        m_game->back(- n);
+    }
     else
-	{
-	    int n=0;
-	    debug("MainFrame::Play +%d\n",n);
+    {
+        int n = 0;
+        debug("MainFrame::Play +%d\n",n);
 #ifdef ENABLE_RESLIST_IN_MAIN
-	    n = reslist->GetSelected();
+        n = reslist->GetSelected();
 #else
-	    n = ((ResultFrame*)auxframes_ptr[ ID_Frame_Result ])->GetSelected();
+        n = ((ResultFrame*)auxframes_ptr[ ID_Frame_Result ])->GetSelected();
 #endif
-	    if (n > -1)
-		{
-		    ((Training*)m_game)->playResult(n);
-		}
-	}
+        if (n > -1)
+        {
+            static_cast<Training*>(m_game)->playResult(n);
+        }
+    }
     wxString r = wxU(m_game->getCurrentPlayer().getCurrentRack().toString().c_str());
     rack->SetValue(r);
     UpdateFrames();
@@ -1047,9 +1052,5 @@ MainFrame::TestPlay(int n)
 /****************************************************************/
 /****************************************************************/
 
-/// Local Variables:
-/// mode: c++
-/// mode: hs-minor
-/// c-basic-offset: 4
-/// indent-tabs-mode: nil
-/// End:
+/// Local Variables: / mode: c++ / mode: hs-minor / c-basic-offset: 4 /
+//indent-tabs-mode: nil / End:
